using System;
using Microsoft.SPOT.Hardware;
using System.Threading;
/*
 * Copyright 2013 David Zimmer (http://www.sandsprite.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
namespace Toolbox.Sandsprite.Netduino
{
    /// <summary>Generic KeyPad driver</summary>
    public class MatrixKeyPad : IDisposable
    {
        
        public delegate void KeyPressEvent(uint row, uint col, uint KeyCode, DateTime stamp);
        public KeyPressEvent KeyHandler;
        public bool Enabled;
        public int Debounce = 500;

        private InputPort[] _ColPins;
        private OutputPort[] _RowPins;
        private uint[] _ColPinIds;

        public MatrixKeyPad(Cpu.Pin[] RowPins, Cpu.Pin[] ColPins, KeyPressEvent kpe)
        {

            KeyHandler = kpe;

            this._RowPins = new OutputPort[RowPins.Length];
            for (var i = 0; i < RowPins.Length; ++i)
            {
                this._RowPins[i] = new OutputPort(RowPins[i], true);
            }
            
            this._ColPinIds = new uint[ColPins.Length];
            this._ColPins = new InputPort[ColPins.Length];
            for (var i = 0; i < ColPins.Length; ++i)
            {
                this._ColPinIds[i] = (uint)ColPins[i];
                this._ColPins[i] = new InputPort(ColPins[i], true, Port.ResistorMode.PullUp);
            }
        }

        public void StartScanLoop()
        {
            Enabled = true;
            while (true)
            {
                if (Enabled) ScanKeys();
                Thread.Sleep(10);
            }
        }

        private void ScanKeys()
        {
            for (uint Row = 0; Row < this._RowPins.Length; Row++)
            {
                _RowPins[Row].Write(false); //set this row low
                //Thread.Sleep(50); //give the 2nd controller a chance to detect the input change (cheating)
                for (uint Column = 0; Column < _ColPinIds.Length; Column++)
                {
                    if ( isLow(_ColPins[Column]) ) //see is this column has drawn low
                    {
                        uint keyIndex = unchecked((uint)(Row * _ColPins.Length + Column));
                        while ( isLow(_ColPins[Column]) ) Thread.Sleep(5); //wait for key release before we raise event
                        this.KeyHandler(Row, Column, keyIndex, DateTime.Now);
                        //Thread.Sleep(this.Debounce);
                    }
                    if (!this.Enabled) return;
                }
                this._RowPins[Row].Write(true); //tests are over for this row, restore high state...
                if (!this.Enabled) return;
            }
        }

        public bool isLow(InputPort ip) { return !ip.Read(); }
        public void Disable() { this.Enabled = false; }
        public void Enable() { this.Enabled = true; }

        public void Dispose()
        {
            for (int i = 0; i < this._ColPins.Length; ++i) this._ColPins[i].Dispose();
            for (int i = 0; i < this._RowPins.Length; ++i) this._RowPins[i].Dispose();
        }

    }
}
